; test program for reading/writing

disk0 equ 0
track equ 1
sector equ 18
offset equ 20
data equ 'A'

dcx sp       ; set stack

mvi a, disk0 ; select drive
out 08h

call ltrack  ; step in 'track' times

; *** WRITE DATA ***
call we      ; write enable
call lsector ; load sector
call loffset ; load offset
call write

; *** READ DATA ***
call lsector ; load sector
call loffset ; load offset
call read
lxi h, readdata
mov m, a

hlt

ltrack0:     ; load track 0
in 08h       ; get drive status
ani 1000000b ; track 0 ?
rz           ; yes, return
mvi a, 1000b ; head unload
out 09h
call movetrk ; wait until we can move track
mvi a, 10b   ; step out track
out 08h
jmp ltrack0

ltrack:      ; step in track 'track' times
call ltrack0 ; load track 0
mvi b, track+1
stepin:
dcr b        ; decrement b
rz           ; end reached ? if yes, return
call movetrk
mvi a, 1
out 09h
jmp stepin

movetrk:     ; wait until we can move track
in 08h       ; get status
ani 10b      ; can move track ?
jnz movetrk  ; no, wait until can
ret

lsector:     ; load sector
mvi a, 100b  ; head load
out 09h
waits:
in 09h       ; get sector number
ani 3Fh      ; clear unused bits
rrc          ; rotate right
cpi sector
jnz waits
ret          ; sector loaded

loffset:     ; load offset
mvi b, offset+1
stepoff:
dcr b        ; decrement b
rz           ; end reached ? if yes, return
call read    ; read data => increment offset
jmp stepoff

read:
in 08h       ; get status
ani 100b     ; head must be loaded
rnz          ; return if it is not
waitr:
in 08h       ; get status
ani 10000000b ; new read data available ?
jnz waitr
in 0Ah       ; read data
ret

we:
mvi a, 10000000b ; write enable
out 09h
ret

write:       ; write data in b register to disk
in 08h       ; get status
ani 100b     ; head must be loaded
rnz          ; return if it is not
waitw:
in 08h       ; get status
ani 1        ; enter new write data ?
jnz waitw    ; if not, wait until yes
mvi a, data
out 0Ah      ; write data
ret

org 200h
readdata: db 0
